<?php
/**
 * Inventory addon for Interspire Shopping Cart
 *
 * Manage inventory, see trends and reorder dates
 *
 * @author Mike Kruckenberg, Josaze Software
 *
 * Copyright 2010 Josaze Software
 *
 * Version 1.0
 */
class ADDON_INVENTORY extends ISC_ADDON
{
	
	private $trackSQL;

	public function __construct()
	{
		// Call the parent constructor (this is required!)
		parent::__construct();
		
		// Set to true if you wish to log SQL calls (and line called from) in the store log
		$this->trackSQL = false;
		//$this->trackSQL = true;
		
		// Set the addon ID
		$this->SetId('addon_inventory');

		// Set the display name for this addon
		$this->SetName(GetLang('InventoryAddonName'));

		// Set the image for this addon
		$this->SetImage('');

		// Set the help text for this addon
		$this->SetHelpText(GetLang('InventoryAddonHelp'));

		// Register a menu item for this addon under the orders menu
		$this->RegisterMenuItem(array(
			'location'		=> 'mnuProducts',
			'icon'			=> 'icon.gif',
			'text'			=> GetLang('InventoryAddonName'),
			'description'	=> GetLang('InventoryAddonDescription'),
			'id' 			=> 'addon_inventory'
		));

	}

	/**
	 * Setup the settings for this addon.
	 */
	public function SetCustomVars()
	{
		// Register the 'WeeksToAverage' setting - a drop down menu
		$this->_variables['WeeksToAverage'] = array(
			'type' => 'dropdown',
			'name' => GetLang('WeeksToAverage'),
            'help' => GetLang('WeeksToAverageHelp'),
			'default' => '1',
			'options' => array(
				1 => '1', 2 => '2', 3 => '3', 4 => '4', 5 => '5', 6 => '6', 7 => '7', 8 => '8',
				9 => '9', 10 => '10', 11 => '11', 12 => '12', 13 => '13', 14 => '14', 15 => '15',
				16 => '16', 17 => '17', 18 => '18', 19 => '19', 20 => '20', 21 => '21', 22 => '22',
				23 => '23', 24 => '24', 25 => '25', 26 => '26'
			),
			'required' => true
		);

		// Register the 'InventoryWeeksInStock' setting - a drop down menu
		$this->_variables['InventoryHoldingWeeks'] = array(
			'type' => 'dropdown',
			'name' => GetLang('InventoryHoldingWeeks'),
            'help' => GetLang('InventoryHoldingWeeksHelp'),
			'default' => '1',
			'options' => array(
				1 => '1', 2 => '2', 3 => '3', 4 => '4', 5 => '5', 6 => '6', 7 => '7', 8 => '8',
				9 => '9', 10 => '10', 11 => '11', 12 => '12', 13 => '13', 14 => '14', 15 => '15',
				16 => '16', 17 => '17', 18 => '18', 19 => '19', 20 => '20', 21 => '21', 22 => '22',
				23 => '23', 24 => '24', 25 => '25', 26 => '26'
			),
			'required' => true
		);

		// Register the 'InventoryWeeksToReplenish' setting - a drop down menu
		$this->_variables['InventoryLeadWeeks'] = array(
			'type' => 'dropdown',
			'name' => GetLang('InventoryLeadWeeks'),
			'help' => GetLang('InventoryLeadWeeksHelp'),
			'default' => '1',
			'options' => array(
				1 => '1', 2 => '2', 3 => '3', 4 => '4', 5 => '5', 6 => '6', 7 => '7', 8 => '8',
				9 => '9', 10 => '10', 11 => '11', 12 => '12', 13 => '13', 14 => '14', 15 => '15',
				16 => '16', 17 => '17', 18 => '18', 19 => '19', 20 => '20', 21 => '21', 22 => '22',
				23 => '23', 24 => '24', 25 => '25', 26 => '26'
			),
			'required' => true
		);
	}

	/**
	 * Initialise any addon specific code that should be run when displaying the settings for this addon.
	 */
	public function Init()
	{
	}

	/**
	 * The main entry point for this addon. In this case, show a template asking for user input.
	 */
	public function EntryPoint()
	{

        $query = "select productid,prodname,prodcurrentinv,proddateadded from [|PREFIX|]products where prodinvtrack != 0 order by prodname";
        $result = $this->inventory_trapSQL(__LINE__, $query);
        
        // get the time for inventory  the unix time 
        $to_avg_weeks = $this->GetValue('WeeksToAverage');
        $to_avg_time = $to_avg_weeks * 7 * 24 * 60 * 60;
        $holding_weeks = $this->GetValue('InventoryHoldingWeeks');
        $lead_time = $this->GetValue('InventoryLeadWeeks') * 7 * 24 * 60 * 60;

        $GLOBALS['InventoryStatus'] = '';
        while($row = $GLOBALS['ISC_CLASS_DB']->Fetch($result)) {
            $GLOBALS['ProductID'] = $row['productid'];
            $GLOBALS['ProductName'] = $row['prodname'];
            $GLOBALS['ProductCurrentInv'] = $GLOBALS['QtyCommitted'] = '';
            
            /*
             *  If the product was added later than the averaging period specified adjust to use
             *  the full period the product has been available and avoid skewing the results
             */ 
            if($row['proddateadded'] > (time() - $to_avg_time)) {
            	$this_to_avg_time = time() - $row['proddateadded'];
            	$this_to_avg_weeks = $this_to_avg_time / (7*24*60*60);
            }
            else {
            	$this_to_avg_time = $to_avg_time;
            	$this_to_avg_weeks = $to_avg_weeks;
            }
            
//            $this_to_avg_time = $to_avg_time;
//            $this_to_avg_weeks = $to_avg_weeks;

            // check if there are variations for this product
            $var_query = "select vcoptionids,vcvariationid,vcstock from [|PREFIX|]product_variation_combinations where vcproductid = ".$row['productid'];
            $var_result = $this->inventory_trapSQL(__LINE__, $var_query);

            // there are variations
            if ($GLOBALS['ISC_CLASS_DB']->CountResult($var_result) > 0) {
                $GLOBALS['QtySold'] = $GLOBALS['QtyPend'] = $GLOBALS['DepletedOn'] = $GLOBALS['OrderPoint'] = $GLOBALS['OrderQty'] = '';
                $GLOBALS['InventoryStatus'] .= $this->ParseTemplate('inventory.row',true);
                $GLOBALS['ProductID'] = '';
                while($vrow = $GLOBALS['ISC_CLASS_DB']->Fetch($var_result)) {
                    $GLOBALS['ProductName'] = '';
                    $value_cond = array();
                    $opt_query = "select voname,vovalue from [|PREFIX|]product_variation_options where voptionid in (".$vrow['vcoptionids'].")";
                    $opt_result = $this->inventory_trapSQL(__LINE__, $opt_query);
                    $count_where = '';
                    while($orow = $GLOBALS['ISC_CLASS_DB']->Fetch($opt_result)) {
                        $GLOBALS['ProductName'] .= $orow['vovalue']." ";
                        $count_where .= " and ordprodoptions rlike '".$orow['vovalue']."'";
                    }

                    // get the number of committed products
                    $committed_query = "select sum(ordprodqty) as soldCount from [|PREFIX|]order_products op, [|PREFIX|]orders o where op.orderorderid = o.orderid and o.ordstatus = 11 and ordprodid = ".$row['productid'].$count_where;
                    $committed_result = $this->inventory_trapSQL(__LINE__, $committed_query);
                    $committed_row = $GLOBALS['ISC_CLASS_DB']->Fetch($committed_result);
                    $GLOBALS['QtyCommitted'] = $committed_row['soldCount'];

                    // get the number of products sold over the specified period of time
                    $sold_query = "select sum(ordprodqty) as soldCount from [|PREFIX|]order_products op, [|PREFIX|]orders o where op.orderorderid = o.orderid and orddate > UNIX_TIMESTAMP(now()) - $this_to_avg_time and o.ordstatus != 4 and o.ordstatus != 5 and ordprodid = ".$row['productid'].$count_where;
                    $sold_result = $this->inventory_trapSQL(__LINE__, $sold_query);
                    $srow = $GLOBALS['ISC_CLASS_DB']->Fetch($sold_result);
                    $GLOBALS['QtySold'] = ceil($srow['soldCount']/$this_to_avg_weeks);


                    /*
                     * If we haven't sold anything within the averaging period we can't
                     * determine when it will run out or what quantity will need to be
                     * ordered.
                     */
                    if($GLOBALS['QtySold'] <= 0) {
						$GLOBALS['QtySold'] = 0;
						$GLOBALS['DepletedOn'] = 'Unknown';
						$GLOBALS['OrderPoint'] = 'Unknown';
						$GLOBALS['OrderQty'] = '0';
					}
					else {
	                    $out_in_secs = ceil((($vrow['vcstock'] - $GLOBALS['QtyPend'])/$GLOBALS['QtySold'])*7*24*60*60);
	                    $out_date = isc_date(GetConfig('DisplayDateFormat'),time() + $out_in_secs);
    	                $GLOBALS['DepletedOn'] = $out_date;

        	            $order_on_time = time() + $out_in_secs - $lead_time;
            	        $GLOBALS['OrderPoint'] = isc_date(GetConfig('DisplayDateFormat'),$order_on_time);
                	    if (time() > $order_on_time) {
                    	    $GLOBALS['OrderPoint'] = "<span style='padding: 3px; background: yellow;'>".$GLOBALS['OrderPoint']."</span>";
	                    }

    	                $GLOBALS['OrderQty'] = $GLOBALS['QtySold'] * $holding_weeks;
					}

                    $GLOBALS['ProductCurrentInv'] = $vrow['vcstock'];
                    $GLOBALS['InventoryStatus'] .= $this->ParseTemplate('inventory.subrow',true);
                }

            } else {
            	
            	//TODO is this fix required?
            	$GLOBALS['QtyPend'] = 0;

                // get the number of committed products
                $committed_query = "select sum(ordprodqty) as soldCount from [|PREFIX|]order_products op, [|PREFIX|]orders o where op.orderorderid = o.orderid and o.ordstatus = 11 and ordprodid = ".$row['productid'];
                $committed_result = $this->inventory_trapSQL(__LINE__, $committed_query);
                $committed_row = $GLOBALS['ISC_CLASS_DB']->Fetch($committed_result);
                $GLOBALS['QtyCommitted'] = $committed_row['soldCount'];

                // get the number of products sold over the specified period of time
                $sold_query = "select sum(ordprodqty) as soldCount from [|PREFIX|]order_products op, [|PREFIX|]orders o where op.orderorderid = o.orderid and orddate > UNIX_TIMESTAMP(now()) - $this_to_avg_time and o.ordstatus != 4 and o.ordstatus != 5 and ordprodid = ".$row['productid'];
                $sold_result = $this->inventory_trapSQL(__LINE__, $sold_query);
                $srow = $GLOBALS['ISC_CLASS_DB']->Fetch($sold_result);
                $GLOBALS['QtySold'] = ceil($srow['soldCount']/$this_to_avg_weeks);
                
                if($GLOBALS['QtySold'] <= 0) {
						$GLOBALS['QtySold'] = 0;
						$GLOBALS['DepletedOn'] = 'Unknown';
						$GLOBALS['OrderPoint'] = 'Unknown';
						$GLOBALS['OrderQty'] = '0';
					}
				else {
	                $out_in_secs = ceil((($row['prodcurrentinv'] - $GLOBALS['QtyPend'])/$GLOBALS['QtySold'])*7*24*60*60);
    	            $GLOBALS['DepletedOn'] = isc_date(GetConfig('DisplayDateFormat'),time() + $out_in_secs);

        	        $order_on_time = time() + $out_in_secs - $lead_time;
            	    $GLOBALS['OrderPoint'] = isc_date(GetConfig('DisplayDateFormat'),$order_on_time);
                	if (time() > $order_on_time) {
                    	$GLOBALS['OrderPoint'] = "<span style='padding: 3px; background: yellow;'>".$GLOBALS['OrderPoint']."</span>";
	                }

    	            $GLOBALS['OrderQty'] = $GLOBALS['QtySold'] * $holding_weeks;
				}

                $GLOBALS['ProductCurrentInv'] = $row['prodcurrentinv'];
                $GLOBALS['InventoryStatus'] .= $this->ParseTemplate('inventory.row',true);
            }
        }

		$this->ParseTemplate('inventory');
	}
	
	/*********************************************
	 * Help with debugging
	 */
		public function inventory_trapSQL($line, $sql) {
			$errorDetails = '';
			if(!is_array($sql)) {
				$queries[0] = $sql;
			}
			else {
				$queries = $sql;
			}
			foreach($queries AS $query) {
				if($this->trackSQL) {
					$errorDetails .= "[Line: $line] $query<br />\n";
				}
				// Run the queries we generated for updating component part products
				$result = $GLOBALS['ISC_CLASS_DB']->Query($query);

				if($this->trackSQL) {
					//echo "SQL: ".$query."<br />\n";
					$GLOBALS['ISC_CLASS_LOG']->LogSystemDebug('php', 'DEBUG SQL', $errorDetails);
				}
				return $result;
			}
		}

}
